import React from 'react';
import ReactDOM from 'react-dom';
import TestUtils from 'react-addons-test-utils';
import purebem from 'purebem';
import { mount } from 'enzyme';

import Datepicker from './Datepicker';
import datepickerState from './redux/datepickerState';


const block = purebem.of('test-datepicker');
const controlElement = block('control');

const TestDatepicker = Datepicker({
    block,
    displayName: 'TestDatepicker'
});

describe('Datepicker', () => {

    it('should render empty text if no options', () => {
        const wrapper = mount(
            <TestDatepicker
                emptyText="Empty" />
        );

        expect(wrapper.text()).toBe('Empty');
    });

    [
        { name: 'should render date if passed as Date', data: { value: new Date(2000, 0, 1) } },
        { name: 'should render date if passed as timestamp', data: { value: 946681200000 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const wrapper = mount(
                <TestDatepicker
                    { ...datepickerState({ value: fixture.data.value }) } />
            );

            expect(wrapper.text()).toBe('2000-01-01');
        });
    });

    it('should expand collapsed datepicker when clicked', () => {
        const onExpand = jest.fn();

        const datepicker = TestUtils.renderIntoDocument(
            <TestDatepicker
                { ...datepickerState({ value: new Date(2000, 0, 1) }) }
                onExpand={ onExpand } />
        );

        const node = TestUtils.findRenderedDOMComponentWithClass(datepicker, controlElement);
        TestUtils.Simulate.click(node);

        expect(onExpand).toHaveBeenCalled();
    });

    [
        { name: 'should expand collapsed datepicker when using enter', data: { which: 13 } },
        { name: 'should expand collapsed datepicker when using space', data: { which: 32 } },
        { name: 'should expand collapsed datepicker when using down arrow', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onExpand = jest.fn();

            const dropdown = TestUtils.renderIntoDocument(
                <TestDatepicker
                    { ...datepickerState({ value: new Date(2000, 0, 1) }) }
                    onExpand={ onExpand } />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onExpand).toHaveBeenCalled();
        });
    });

    it('should not expand collapsed datepicked when click and disabled', () => {
        const onExpand = jest.fn();

        const datepicker = TestUtils.renderIntoDocument(
            <TestDatepicker
                { ...datepickerState({ value: new Date(2000, 0, 1) }) }
                disabled={ true }
                onExpand={ onExpand } />
        );

        const node = TestUtils.findRenderedDOMComponentWithClass(datepicker, controlElement);
        TestUtils.Simulate.click(node);

        expect(onExpand).not.toHaveBeenCalled();
    });

    it('should collapse expanded dropdown when clicked', () => {
        const onCollapse = jest.fn();

        const datepicker = TestUtils.renderIntoDocument(
            <TestDatepicker
                { ...datepickerState({ expanded: true, value: new Date(2000, 0, 1) }) }
                onCollapse={ onCollapse } />
        );

        const node = TestUtils.findRenderedDOMComponentWithClass(datepicker, controlElement);
        TestUtils.Simulate.click(node);

        expect(onCollapse).toHaveBeenCalled();
    });

    it('should collapse expanded dropdown when clicking outside', () => {
        const onCollapse = jest.fn();

        TestUtils.renderIntoDocument(
            <TestDatepicker
                { ...datepickerState({ expanded: true, value: new Date(2000, 0, 1) }) }
                onCollapse={ onCollapse } />
        );

        document.dispatchEvent(new Event('mousedown'));
        expect(onCollapse).toHaveBeenCalled();
    });

    it('should collapse expanded dropdown when using escape', () => {
        const onCollapse = jest.fn();

        const datepicker = TestUtils.renderIntoDocument(
            <TestDatepicker
                { ...datepickerState({ expanded: true, value: new Date(2000, 0, 1) }) }
                onCollapse={ onCollapse } />
        );

        const node = ReactDOM.findDOMNode(datepicker);
        TestUtils.Simulate.keyDown(node, { which: 27 });

        expect(onCollapse).toHaveBeenCalled();
    });

    [
        { name: 'should go to previous date when using left arrow when expanded', data: { which: 37 }, expected: { selectedIndex: +new Date(2000, 0, 18) } },
        { name: 'should go to previous week when using up arrow when expanded', data: { which: 38 }, expected: { selectedIndex: +new Date(2000, 0, 12) } },
        { name: 'should go to next date when using right arrow when expanded', data: { which: 39 }, expected: { selectedIndex: +new Date(2000, 0, 20) } },
        { name: 'should go to next week when using down arrow when expanded', data: { which: 40 }, expected: { selectedIndex: +new Date(2000, 0, 26) } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onSelectedIndexChange = jest.fn();

            const datepicker = TestUtils.renderIntoDocument(
                <TestDatepicker
                    { ...datepickerState({ expanded: true, value: +new Date(2000, 0, 19) }) }
                    onSelectedIndexChange={ onSelectedIndexChange } />
            );

            const node = ReactDOM.findDOMNode(datepicker);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onSelectedIndexChange).toHaveBeenCalledWith(+new Date(2000, 0, 1), fixture.expected.selectedIndex);
        });
    });

    [
        { name: 'should go to previous date/month when using left arrow when expanded', data: { which: 37 }, expected: { currentMonth: +new Date(2000, 0, 1), selectedIndex: +new Date(2000, 0, 29) } },
        { name: 'should go to previous week/month when using up arrow when expanded', data: { which: 38 }, expected: { currentMonth: +new Date(2000, 0, 1), selectedIndex: +new Date(2000, 0, 23) } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onSelectedIndexChange = jest.fn();

            const datepicker = TestUtils.renderIntoDocument(
                <TestDatepicker
                    { ...datepickerState({ expanded: true, value: +new Date(2000, 1, 1) }) }
                    onSelectedIndexChange={ onSelectedIndexChange }
                    selectedIndex={ +new Date(2000, 0, 30) } />
            );

            const node = ReactDOM.findDOMNode(datepicker);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onSelectedIndexChange).toHaveBeenCalledWith(fixture.expected.currentMonth, fixture.expected.selectedIndex);
        });
    });

    [
        { name: 'should go to next date/month when using right arrow when expanded', data: { which: 39 }, expected: { currentMonth: +new Date(2000, 1, 1), selectedIndex: +new Date(2000, 1, 6) } },
        { name: 'should go to next week/month when using down arrow when expanded', data: { which: 40 }, expected: { currentMonth: +new Date(2000, 1, 1), selectedIndex: +new Date(2000, 1, 12) } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const onSelectedIndexChange = jest.fn();

            const datepicker = TestUtils.renderIntoDocument(
                <TestDatepicker
                    { ...datepickerState({ expanded: true, value: +new Date(2000, 0, 31) }) }
                    onSelectedIndexChange={ onSelectedIndexChange }
                    selectedIndex={ +new Date(2000, 1, 5) } />
            );

            const node = ReactDOM.findDOMNode(datepicker);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(onSelectedIndexChange).toHaveBeenCalledWith(fixture.expected.currentMonth, fixture.expected.selectedIndex);
        });
    });

    it('should go to next month when using tab key when expanded', () => {
        const onSelectedIndexChange = jest.fn();

        const datepicker = TestUtils.renderIntoDocument(
            <TestDatepicker
                { ...datepickerState({ expanded: true, value: +new Date(2000, 0, 19) }) }
                onSelectedIndexChange={ onSelectedIndexChange } />
        );

        const node = ReactDOM.findDOMNode(datepicker);
        TestUtils.Simulate.keyDown(node, { which: 9 });

        expect(onSelectedIndexChange).toHaveBeenCalledWith(+new Date(2000, 1, 1), +new Date(2000, 1, 19));
    });

    it('should go to previous month when using shift + tab key when expanded', () => {
        const onSelectedIndexChange = jest.fn();

        const datepicker = TestUtils.renderIntoDocument(
            <TestDatepicker
                { ...datepickerState({ expanded: true, value: +new Date(2000, 0, 19) }) }
                onSelectedIndexChange={ onSelectedIndexChange } />
        );

        const node = ReactDOM.findDOMNode(datepicker);
        TestUtils.Simulate.keyDown(node, { shiftKey: true, which: 9 });

        expect(onSelectedIndexChange).toHaveBeenCalledWith(+new Date(1999, 11, 1), +new Date(1999, 11, 19));
    });

});
